/*
 * FILE --- observer.hh
 *
 * PURPOSE --- Header file for a class that implements a Linear State 
 * Observer for Single-Input-Single-Output (SISO) plants.
 * The user specifies the dimension of the state of the observer and the 
 * observer gains. For the simulation of the dynamics the utility uses a
 * 4'th order Runge-Kutta simulator.
 *
 */

// Prevent Multiple Inclusion
#ifndef OBSERVER_HH
#define OBSERVER_HH

// Include Files
#include <stdlib.h>
#include "rk4_sim.hh"

// Local Definitions
#define OBSERVER_NO_ERROR 0
#define OBSERVER_MEM_ERROR 1

class OBSERVER: public RK4_SIM
{
public:

  // Constructor
  OBSERVER(int _plant_order, double _sample_time = 0.01);

  // Destructor
  ~OBSERVER();

  // Initialize the observer parameters
  void init_observer(double _sample_time, 
		     int _sim_per_sample,
		     double _initial_time = 0.0, 
		     double *_initial_state = NULL);
  
  // Set the gains of the observer
  // _observer_gains = an array of size plant_order specifying the observer 
  // gains
  // if _observer_gains == NULL then all the gains are initialized to zero
  void set_gains(double *_observer_gains = NULL);

  // Set or return the value of a particular gain
  void set_gain(int _index, double _val) {observer_gains[_index] = _val;};
  double get_gain(int _index) {return observer_gains[_index];};


  // Overload DERIV function of RK4_SIM for the simulator
  // This function should contain the dynamics of the observer
  virtual void DERIV(const double t, const double *x, 
		     const double *u, double *xdot);

  int observer_err;  // Variable to hold the error condition

protected:
  int sim_per_sample; // # of simulations per sample time

private:
  double sample_time;  // Sampling time
  double *observer_gains; // An array to hold observer gains

  // Allocate/free memory for the gains
  void allocate_memory(void);
  void free_memory(void);
};

#endif // OBSERVER_HH
